@@ -6,6 +6,7 @@ import warnings as w
 import numbers
 import pandas as pd
 from Thermobar.core import *
+from tqdm import tqdm
 
 
 ## Equations: Plag-Liquid thermometers
@@ -88,7 +89,7 @@ Kspar_Liq_T_funcs_by_name = {p.__name__: p for p in Kspar_Liq_T_funcs}
 
 def calculate_fspar_liq_temp(*, plag_comps=None, kspar_comps=None, meltmatch_plag=None,
 meltmatch_kspar=None,
-    liq_comps=None, equationT=None, P=None, H2O_Liq=None, eq_tests=False):
+    liq_comps=None, equationT=None, P=None, H2O_Liq=None, eq_tests=False, warnAn=False):
     '''
     Liquid-Feldspar thermometery (same function for Plag and Kspar),
     returns temperature in Kelvin.
@@ -133,6 +134,17 @@ meltmatch_kspar=None,
             If eq_tests is True
 
     '''
+    if kspar_comps is not None:
+        kspar_comps_c=kspar_comps.copy()
+        kspar_comps_c=kspar_comps_c.reset_index(drop=True)
+    if plag_comps is not None:
+        plag_comps_c=plag_comps.copy()
+        plag_comps_c=plag_comps_c.reset_index(drop=True)
+    if liq_comps is not None:
+        liq_comps_c=liq_comps.copy()
+        liq_comps_c=liq_comps_c.reset_index(drop=True)
+        if H2O_Liq is not None:
+            liq_comps_c['H2O_Liq'] = H2O_Liq
 
 
     if plag_comps is not None or meltmatch_plag is not None:
@@ -150,10 +162,7 @@ meltmatch_kspar=None,
             raise ValueError(f'{equationT} is not a valid equation for Kspar-Liquid') from None
         sig=inspect.signature(func)
 
-    if meltmatch_plag is None and meltmatch_kspar is None:
-        liq_comps_c = liq_comps.copy()
-        if H2O_Liq is not None:
-            liq_comps_c['H2O_Liq'] = H2O_Liq
+
 
     if sig.parameters['P'].default is not None:
         if P is None:
@@ -165,11 +174,12 @@ meltmatch_kspar=None,
 
 
     if plag_comps is not None:
-        cat_plags = calculate_cat_fractions_plagioclase(plag_comps=plag_comps)
+        cat_plags = calculate_cat_fractions_plagioclase(plag_comps=plag_comps_c)
         cat_liqs = calculate_anhydrous_cat_fractions_liquid(liq_comps_c)
         combo_fspar_Liq = pd.concat([cat_plags, cat_liqs], axis=1)
-        if np.min(combo_fspar_Liq['An_Plag'] < 0.05):
-            w.warn('Some inputted feldspars have An<0.05, but you have selected a plagioclase-liquid thermometer'
+        if any(combo_fspar_Liq['An_Plag'] < 0.05):
+            if warnAn is True:
+                w.warn('Some inputted feldspars have An<0.05, but you have selected a plagioclase-liquid thermometer'
             '. If these are actually alkali felspars, please use T_P2008_eq24b or T_P2008_24c instead', stacklevel=2)
         Kd_Ab_An = (combo_fspar_Liq['Ab_Plag'] * combo_fspar_Liq['Al_Liq_cat_frac'] * combo_fspar_Liq['Ca_Liq_cat_frac'] /
                     (combo_fspar_Liq['An_Plag'] * combo_fspar_Liq['Na_Liq_cat_frac'] * combo_fspar_Liq['Si_Liq_cat_frac']))
@@ -177,7 +187,7 @@ meltmatch_kspar=None,
 
 
     if kspar_comps is not None:
-        cat_kspars = calculate_cat_fractions_kspar(kspar_comps=kspar_comps)
+        cat_kspars = calculate_cat_fractions_kspar(kspar_comps=kspar_comps_c)
         cat_liqs = calculate_anhydrous_cat_fractions_liquid(liq_comps_c)
         combo_fspar_Liq = pd.concat([cat_kspars, cat_liqs], axis=1)
 
@@ -214,8 +224,8 @@ meltmatch_kspar=None,
             print('Sorry, no equilibrium tests implemented for Kspar-Liquid')
             return T_K
         if plag_comps is not None:
-            eq_tests=calculate_plag_liq_eq_tests(liq_comps=liq_comps,
-            plag_comps=plag_comps, P=P, T=T_K)
+            eq_tests=calculate_plag_liq_eq_tests(liq_comps=liq_comps_c,
+            plag_comps=plag_comps_c, P=P, T=T_K)
             eq_tests.insert(0, "T_K_calc", T_K)
             return eq_tests
 
@@ -271,6 +281,18 @@ def calculate_fspar_liq_press(*, plag_comps=None, kspar_comps=None, liq_comps=No
 
 
     '''
+    if kspar_comps is not None:
+        kspar_comps_c=kspar_comps.copy()
+        kspar_comps_c=kspar_comps_c.reset_index(drop=True)
+    if plag_comps is not None:
+        plag_comps_c=plag_comps.copy()
+        plag_comps_c=plag_comps_c.reset_index(drop=True)
+    if liq_comps is not None:
+        liq_comps_c=liq_comps.copy()
+        liq_comps_c=liq_comps_c.reset_index(drop=True)
+        if H2O_Liq is not None:
+            liq_comps_c['H2O_Liq'] = H2O_Liq
+
     if kspar_comps is not None:
         raise TypeError('Sorry, this tool doesnt contain any alkali-fspar-liquid barometers,'
         ' this option is here as a place holder incase a great one comes along!')
@@ -283,13 +305,12 @@ def calculate_fspar_liq_press(*, plag_comps=None, kspar_comps=None, liq_comps=No
 
 
 
-    liq_comps_c = liq_comps.copy()
-    if H2O_Liq is not None:
-        liq_comps_c['H2O_Liq'] = H2O_Liq
 
 
-    cat_plags = calculate_cat_fractions_plagioclase(plag_comps=plag_comps)
-    cat_liqs = calculate_anhydrous_cat_fractions_liquid(liq_comps)
+
+
+    cat_plags = calculate_cat_fractions_plagioclase(plag_comps=plag_comps_c)
+    cat_liqs = calculate_anhydrous_cat_fractions_liquid(liq_comps_c)
     combo_plag_liq = pd.concat([cat_plags, cat_liqs], axis=1)
 
     Kd_Ab_An = (combo_plag_liq['Ab_Plag'] * combo_plag_liq['Al_Liq_cat_frac'] * combo_plag_liq['Ca_Liq_cat_frac'] /
@@ -323,8 +344,8 @@ def calculate_fspar_liq_press(*, plag_comps=None, kspar_comps=None, liq_comps=No
         return P_kbar
     if eq_tests is True:
 
-        eq_tests=calculate_plag_liq_eq_tests(liq_comps=liq_comps,
-        plag_comps=plag_comps, P=P_kbar, T=T)
+        eq_tests=calculate_plag_liq_eq_tests(liq_comps=liq_comps_c,
+        plag_comps=plag_comps_c, P=P_kbar, T=T)
         eq_tests.insert(1, "P_kbar_calc", P_kbar)
         return eq_tests
     return P_kbar
@@ -333,7 +354,7 @@ def calculate_fspar_liq_press(*, plag_comps=None, kspar_comps=None, liq_comps=No
 
 def calculate_fspar_liq_press_temp(*, liq_comps=None, plag_comps=None, kspar_comps=None,
                                 meltmatch=None, equationP=None, equationT=None, iterations=30, T_K_guess=1300,
-                                H2O_Liq=None):
+                                H2O_Liq=None, eq_tests=False):
     '''
     Solves simultaneous equations for temperature and pressure using
     feldspar-liquid thermometers and barometers. Currently no Kspar barometers exist.
@@ -367,14 +388,25 @@ def calculate_fspar_liq_press_temp(*, liq_comps=None, plag_comps=None, kspar_com
     T_K_guess: int or float (Default = 1300K)
          Initial guess of temperature.
 
+    eq_tests: bool
+
+
     Returns
     ---------
     pandas.DataFrame: Pressure in kbar, Temperature in K + fspar+liq comps (if eq_tests=True)
     '''
     # Gives users flexibility to reduce or increase iterations
-    liq_comps_c=liq_comps.copy()
-    if H2O_Liq is not None:
-        liq_comps_c['H2O_Liq']=H2O_Liq
+
+
+    if plag_comps is not None:
+        plag_comps_c=plag_comps.copy()
+        plag_comps_c=plag_comps_c.reset_index(drop=True)
+    if liq_comps is not None:
+        liq_comps_c=liq_comps.copy()
+        liq_comps_c=liq_comps_c.reset_index(drop=True)
+        if H2O_Liq is not None:
+            liq_comps_c['H2O_Liq'] = H2O_Liq
+
     if kspar_comps is not None:
         raise Exception(
             'Sorry, we currently dont have any kspar barometers coded up, so you cant iteratively solve for Kspars')
@@ -382,9 +414,9 @@ def calculate_fspar_liq_press_temp(*, liq_comps=None, plag_comps=None, kspar_com
 
     if meltmatch is None:
         T_func = calculate_fspar_liq_temp(
-            plag_comps=plag_comps, liq_comps=liq_comps_c, equationT=equationT, P="Solve")
+            plag_comps=plag_comps_c, liq_comps=liq_comps_c, equationT=equationT, P="Solve")
         P_func = calculate_fspar_liq_press(
-            plag_comps=plag_comps, liq_comps=liq_comps_c, equationP=equationP,  T="Solve")
+            plag_comps=plag_comps_c, liq_comps=liq_comps_c, equationP=equationP,  T="Solve")
     if meltmatch is not None:
         T_func = calculate_fspar_liq_temp(
             meltmatch=meltmatch, equationT=equationT)
@@ -411,14 +443,44 @@ def calculate_fspar_liq_press_temp(*, liq_comps=None, plag_comps=None, kspar_com
 
         # Gives users flexibility to add a different guess temperature
 
-            for _ in range(iterations):
-                P_guess = P_func(T_K_guess)
-                T_K_guess = T_func(P_guess)
+        count=0
+        for _ in range(iterations):
+            P_guess = P_func(T_K_guess)
+            T_K_guess = T_func(P_guess)
+            if count==iterations-2:
+                # On the second last step, save the pressure
+                P_out_loop=P_guess.values
+                T_out_loop=T_K_guess.values
+            count=count+1
+
+        DeltaP=P_guess-P_out_loop
+        DeltaT=T_K_guess-T_out_loop
 
     # calculates Kd Fe-Mg if eq_tests="True"
 
-    PT_out = pd.DataFrame(data={'P_kbar_calc': P_guess, 'T_K_calc': T_K_guess})
-    return PT_out
+    else:
+        DeltaP=0
+        DeltaT=0
+
+    if eq_tests is True:
+
+        eq_tests=calculate_plag_liq_eq_tests(liq_comps=liq_comps_c,
+        plag_comps=plag_comps_c, P=P_guess, T=T_K_guess)
+        eq_tests.insert(1, "P_kbar_calc", P_guess)
+        eq_tests.insert(2, "T_K_calc", T_K_guess)
+        eq_tests.insert(3, "Delta_P_kbar_Iter", DeltaP)
+        eq_tests.insert(4, 'Delta_T_K_Iter',DeltaT)
+        return eq_tests
+
+    else:
+         PT_out = PT_out = pd.DataFrame(data={'P_kbar_calc': P_guess,
+                                    'T_K_calc': T_K_guess,
+                                    'Delta_P_kbar_Iter': DeltaP,
+                                    'Delta_T_K_Iter': DeltaT})
+         return PT_out
+
+
+
 
 ## Equations: Plag-Liquid hygrometry
 
@@ -470,6 +532,7 @@ def H_Waters2015(T, *, liq_comps=None, plag_comps=None,
     SEE=+-0.35 wt%
 
     '''
+    T=T+0.000000000001 # This stops it being an integer
     # 1st bit calculates mole fractions in the same way as Waters and Lange.
     # Note, to match excel, all Fe is considered as FeOT.
     mol_prop = calculate_anhydrous_mol_proportions_liquid(liq_comps=liq_comps)
@@ -734,11 +797,21 @@ def calculate_fspar_liq_hygr(*, liq_comps, plag_comps=None, kspar_comps=None,
     if plag_comps is None and XAb is None:
         raise Exception('You must enter either plag_comps=, or an XAb and XAn content')
 
+
+    if plag_comps is not None:
+        plag_comps_c=plag_comps.copy()
+        plag_comps_c=plag_comps_c.reset_index(drop=True)
+    if liq_comps is not None:
+        liq_comps_c=liq_comps.copy()
+        liq_comps_c=liq_comps_c.reset_index(drop=True)
+
+
+
     if equationH == "H_Waters2015":
         if XAn is not None and XAb is not None and plag_comps is None:
             Combo_Out = H_Waters2015(
-                liq_comps=liq_comps, P=P, T=T, XAn=XAn, XAb=XAb)
-            eq_tests=calculate_plag_liq_eq_tests(liq_comps=liq_comps, XAn=XAn, XAb=XAb,
+                liq_comps=liq_comps_c, P=P, T=T, XAn=XAn, XAb=XAb)
+            eq_tests=calculate_plag_liq_eq_tests(liq_comps=liq_comps_c, XAn=XAn, XAb=XAb,
                 P=P, T=T)
 
 
@@ -746,9 +819,9 @@ def calculate_fspar_liq_hygr(*, liq_comps, plag_comps=None, kspar_comps=None,
         if plag_comps is not None:
 
             Combo_Out = H_Waters2015(
-                liq_comps=liq_comps, plag_comps=plag_comps, P=P, T=T)
-            eq_tests=calculate_plag_liq_eq_tests(plag_comps=plag_comps,
-                liq_comps=liq_comps, P=P, T=T)
+                liq_comps=liq_comps_c, plag_comps=plag_comps_c, P=P, T=T)
+            eq_tests=calculate_plag_liq_eq_tests(plag_comps=plag_comps_c,
+                liq_comps=liq_comps_c, P=P, T=T)
 
         eq_tests.insert(0, 'H2O_calc', Combo_Out['H2O_calc'])
 
@@ -761,10 +834,10 @@ def calculate_fspar_liq_hygr(*, liq_comps, plag_comps=None, kspar_comps=None,
             ' because we want to return you eq tests, you need to enter a P')
         if plag_comps is not None:
             combo_plag_liq = calculate_plag_liq_eq_tests(
-                liq_comps=liq_comps, plag_comps=plag_comps, T=T, P=P)
+                liq_comps=liq_comps_c, plag_comps=plag_comps_c, T=T, P=P)
         if plag_comps is None:
             combo_plag_liq = calculate_plag_liq_eq_tests(
-                liq_comps=liq_comps, T=T, P=P, XAn=XAn, XAb=XAb)
+                liq_comps=liq_comps_c, T=T, P=P, XAn=XAn, XAb=XAb)
             combo_plag_liq['An_Plag'] = XAn
             combo_plag_liq['Ab_Plag'] = XAb
 
@@ -785,7 +858,7 @@ def calculate_fspar_liq_hygr(*, liq_comps, plag_comps=None, kspar_comps=None,
 # plag_liq_H_funcs_by_name = {p.__name__: p for p in plag_liq_H_funcs}
 
 def calculate_fspar_liq_temp_hygr(*, liq_comps, plag_comps, equationT, equationH, iterations=20,
-                                P=None, kspar_comps=None, eq_tests=True):
+                                P=None, kspar_comps=None, eq_tests=True, H2O_estimate=1):
 
     '''
     Iterates temperature and water content for Plag-liquid pairs for a user-specified number of
@@ -811,6 +884,8 @@ def calculate_fspar_liq_temp_hygr(*, liq_comps, plag_comps, equationT, equationH
 
         |   T_Put2008_eq23 (Plag-Liq, P-dependent, H2O-dependent)
         |   T_Put2008_eq24a (Plag-Liq, P-dependent, H2O-dependent)
+    H2O_estimate: float, int
+        Initial estimate of H2O content. Can help convergence.
 
     P: float, int, pandas.Series
         Pressure (kbar) to perform calculations at
@@ -853,27 +928,39 @@ def calculate_fspar_liq_temp_hygr(*, liq_comps, plag_comps, equationT, equationH
         raise ValueError(f'{equationH} is not a valid equation') from None
     sig=inspect.signature(func)
 
+    if plag_comps is not None:
+        plag_comps_c=plag_comps.copy()
+        plag_comps_c=plag_comps_c.reset_index(drop=True)
+    if liq_comps is not None:
+        liq_comps_c=liq_comps.copy()
+        liq_comps_c=liq_comps_c.reset_index(drop=True)
+
+
+
 
     if P is None:
         raise ValueError('Please enter a pressure in kbar (P=...)')
 
     H2O_iter=np.empty(len(liq_comps), dtype=float)
-    T_iter_23_W2015=calculate_fspar_liq_temp(liq_comps=liq_comps, plag_comps=plag_comps, equationT=equationT,
-                               P=P, H2O_Liq=0)
+    T_iter_23_W2015=calculate_fspar_liq_temp(liq_comps=liq_comps_c, plag_comps=plag_comps_c, equationT=equationT,
+                               P=P, H2O_Liq=H2O_estimate)
     T_sample=np.empty([len(T_iter_23_W2015), iterations], dtype=float)
     H2O_sample=np.empty([len(T_iter_23_W2015), iterations], dtype=float)
     It_sample=np.empty( iterations)
 
     for i in range (0, iterations):
-        H2O_iter_23_W2015=calculate_fspar_liq_hygr(liq_comps=liq_comps, plag_comps=plag_comps,
+
+        H2O_iter_23_W2015=calculate_fspar_liq_hygr(liq_comps=liq_comps_c, plag_comps=plag_comps_c,
                                             equationH=equationH,
                                                P=P, T=T_iter_23_W2015)
         H2O_sample[:, i]=H2O_iter_23_W2015['H2O_calc']
 
-        T_iter_23_W2015=calculate_fspar_liq_temp(liq_comps=liq_comps, plag_comps=plag_comps, equationT=equationT,
+        T_iter_23_W2015=calculate_fspar_liq_temp(liq_comps=liq_comps_c, plag_comps=plag_comps_c, equationT=equationT,
                                P=P, H2O_Liq=H2O_iter_23_W2015['H2O_calc'])
         T_sample[:, i]=T_iter_23_W2015
         It_sample[i]=i
+
+
     Combined_output=H2O_iter_23_W2015.copy()
     Combined_output.insert(0, '# of iterations', iterations)
     Combined_output.insert(1, 'T_K_calc', T_iter_23_W2015)
@@ -886,17 +973,35 @@ def calculate_fspar_liq_temp_hygr(*, liq_comps, plag_comps, equationT, equationH
 
     # Calculating a dataframe showing the evolution of temperature and H2O vs. number of iterations
     Iter=It_sample
-    for i in range(0, len(liq_comps)):
-        if i==0:
-            T_evol=pd.DataFrame(data={'Iteration': Iter, 'Sample_0_T_calc': T_sample[0, :]})
-            T_evol.insert(i+1, 'Sample_0_H_calc', H2O_sample[0, :])
-        else:
-            new_col_name_T=('Sample_'+str(i)+ "_T_calc")
-            new_col_name_H=('Sample_'+str(i)+ "_H_calc")
-            T_evol.insert(2*i, new_col_name_T, T_sample[i, :])
-            T_evol.insert(2*i+1, new_col_name_H, H2O_sample[i, :])
 
-    return {'T_H_calc': Combined_output, 'T_H_Evolution':  T_evol}
+    df_t=pd.DataFrame(T_sample.T)
+    df_Temp=df_t.add_prefix('Sample_')
+    df_Temp2=df_Temp.add_suffix('_T_calc')
+    df_Temp2
+
+    print(len(df_Temp2))
+
+    df_h=pd.DataFrame(H2O_sample.T)
+    df_H2O=df_h.add_prefix('Sample_')
+    df_H2O2=df_H2O.add_suffix('_H_calc')
+    df_H2O2
+
+    T_evol=pd.concat([df_Temp2, df_H2O2], axis=1)
+    T_evol['Iteration']=Iter
+
+    # for i in range(0, len(liq_comps)):
+    #     if i==0:
+    #         T_evol=pd.DataFrame(data={'Iteration': Iter, 'Sample_0_T_calc': T_sample[0, :]})
+    #         T_evol.insert(i+1, 'Sample_0_H_calc', H2O_sample[0, :])
+    #     else:
+    #         new_col_name_T=('Sample_'+str(i)+ "_T_calc")
+    #         new_col_name_H=('Sample_'+str(i)+ "_H_calc")
+    #         T_evol.insert(2*i, new_col_name_T, T_sample[i, :])
+    #         T_evol.insert(2*i+1, new_col_name_H, H2O_sample[i, :])
+    #
+
+
+    return {'T_H_calc': Combined_output, 'T_H_Evolution':  T_evol, 'T_sample': T_sample, 'H2O_sample': H2O_sample}
 
 
 
@@ -1114,8 +1219,12 @@ def calculate_plag_kspar_temp_matching(*, kspar_comps, plag_comps, equationT=Non
     Combo_plags_kspars_1['K_Barth'] = Combo_plags_kspars_1['Ab_Kspar'] / \
         Combo_plags_kspars_1['Ab_Plag']
     LenCombo = str(np.shape(Combo_plags_kspars)[0])
-    print("Considering " + LenCombo +
-          " Kspar-Plag pairs, be patient if this is >>1 million!")
+    LenPlag=len(plag_comps)
+    LenKspar=len(kspar_comps)
+    print("Considering N=" + str(LenPlag) + " Plag & N=" + str(LenKspar) +" Kspar, which is a total of N="+ str(LenCombo) +
+          " Plag-Kspar pairs, be patient if this is >>1 million!")
+
+
 
     T_K_calc=calculate_plag_kspar_temp(Two_Fspar_Match=Combo_plags_kspars_1.astype(float),
     equationT=equationT, P=P)
@@ -1133,7 +1242,7 @@ def calculate_plag_kspar_temp_matching(*, kspar_comps, plag_comps, equationT=Non
 
 ## Matching functions for fspar-Liq
 def calculate_fspar_liq_temp_hygr_matching(liq_comps, plag_comps, equationT, equationH, iterations=20,
-                                P=None, kspar_comps=None, eq_tests=True):
+                                P=None, kspar_comps=None, Ab_An_P2008=True ):
 
     '''
     Evaluates all possible Plag-liq pairs, iterates T and H2O
@@ -1164,13 +1273,21 @@ def calculate_fspar_liq_temp_hygr_matching(liq_comps, plag_comps, equationT, equ
 
     Returns
     -------
-    pandas.DataFrame
-        T in K for all posible plag-liq matches, along with equilibrium
-        tests, components and input mineral compositions
+    dict
+        'Av_HTs': df of averaged T and H for each Plag, and all the liquids it matches (+eq tests etc)
+        'All_HTs: df of all T and H for all possible Plag-Liq combinations (+eq tests etc)
+        'T_H_Evolution': dataframe of T-H evolution against number of iterations.
+
+
     '''
 
-    if eq_tests is False:
-        print('Too bad, we return the equilibrium tests anyway, as you really need to look at them!')
+
+    if kspar_comps is not None:
+        raise Exception('This function isnt able to use KSpar yet as no hygrometers exist')
+
+    plag_comps_c=plag_comps.copy()
+    liq_comps_c=liq_comps.copy()
+
     #Check valid equation for T
     try:
         func = plag_liq_T_funcs_by_name[equationT]
@@ -1192,16 +1309,24 @@ def calculate_fspar_liq_temp_hygr_matching(liq_comps, plag_comps, equationT, equ
     # computation time
     liq_comps_c=liq_comps.copy()
 
+    plag_comps_c['ID_Fspar'] = plag_comps_c.index
+    liq_comps_c['ID_liq'] = liq_comps_c.index.astype('float64')
+
+    if "Sample_ID_Plag" not in plag_comps:
+        plag_comps_c['Sample_ID_Plag'] = plag_comps.index.astype('float64')
+    if "Sample_ID_liq" not in liq_comps:
+        liq_comps_c['Sample_ID_liq'] = liq_comps.index.astype('float64')
+
 
 
     # This duplicates Plags, repeats liq1-liq1*N, liq2-liq2*N etc.
-    DupFspars = pd.DataFrame(np.repeat(plag_comps.values, np.shape(
+    DupFspars = pd.DataFrame(np.repeat(plag_comps_c.values, np.shape(
         liq_comps_c)[0], axis=0))  # .astype('float64')
-    DupFspars.columns = plag_comps.columns
+    DupFspars.columns = plag_comps_c.columns
 
     # This duplicates liquids like liq1-liq2-liq3 for liq1, liq1-liq2-liq3 for
     # liq2 etc.
-    Dupliqs = pd.concat([liq_comps_c] * np.shape(plag_comps)[0]).reset_index(drop=True)
+    Dupliqs = pd.concat([liq_comps_c] * np.shape(plag_comps_c)[0]).reset_index(drop=True)
     # Combines these merged liquids and liq dataframes
     Combo_fspar_liqs = pd.concat([Dupliqs, DupFspars], axis=1)
 
@@ -1209,8 +1334,13 @@ def calculate_fspar_liq_temp_hygr_matching(liq_comps, plag_comps, equationT, equ
     # Combo_plags_liqs_1['K_Barth'] = Combo_plags_liqs_1['Ab_liq'] / \
     #     Combo_plags_liqs_1['Ab_Plag']
     LenCombo = str(np.shape(Combo_fspar_liqs)[0])
-    print("Considering " + LenCombo +
-          " liq-spar pairs, be patient if this is >>1 million!")
+
+    LenFspar=len(plag_comps)
+    LenLiqs=len(liq_comps)
+    print("Considering N=" + str(LenFspar) + " Fspar & N=" + str(LenLiqs) +" Liqs, which is a total of N="+ str(LenCombo) +
+          " Liq- Fspar pairs, be patient if this is >>1 million!")
+
+
 
     H2O_iter=np.empty(len(Dupliqs), dtype=float)
     T_iter_23_W2015=calculate_fspar_liq_temp(liq_comps=Dupliqs, plag_comps=DupFspars, equationT=equationT,
@@ -1219,16 +1349,28 @@ def calculate_fspar_liq_temp_hygr_matching(liq_comps, plag_comps, equationT, equ
     H2O_sample=np.empty([len(T_iter_23_W2015), iterations], dtype=float)
     It_sample=np.empty( iterations)
 
-    for i in range (0, iterations):
+    for i in tqdm(range(0, iterations)):
+
+
+
         H2O_iter_23_W2015=calculate_fspar_liq_hygr(liq_comps=Dupliqs, plag_comps=DupFspars,
                                             equationH=equationH,
-                                               P=P, T=T_iter_23_W2015)
+                                            P=P, T=T_iter_23_W2015)
         H2O_sample[:, i]=H2O_iter_23_W2015['H2O_calc']
 
-        T_iter_23_W2015=calculate_fspar_liq_temp(liq_comps=Dupliqs, plag_comps=DupFspars, equationT=equationT,
-                               P=P, H2O_Liq=H2O_iter_23_W2015['H2O_calc'])
+        if i==0:
+
+
+            T_iter_23_W2015=calculate_fspar_liq_temp(liq_comps=Dupliqs, plag_comps=DupFspars, equationT=equationT,
+                                P=P, H2O_Liq=H2O_iter_23_W2015['H2O_calc'], warnAn=True)
+        else:
+            T_iter_23_W2015=calculate_fspar_liq_temp(liq_comps=Dupliqs, plag_comps=DupFspars, equationT=equationT,
+                                P=P, H2O_Liq=H2O_iter_23_W2015['H2O_calc'])
+
+
         T_sample[:, i]=T_iter_23_W2015
         It_sample[i]=i
+
     Combined_output=H2O_iter_23_W2015.copy()
     Combined_output.insert(0, '# of iterations', iterations)
     Combined_output.insert(1, 'T_K_calc', T_iter_23_W2015)
@@ -1239,29 +1381,94 @@ def calculate_fspar_liq_temp_hygr_matching(liq_comps, plag_comps, equationT, equ
     Combined_output.insert(2, 'Delta T (last 2 iters)', DeltaT)
     Combined_output.insert(4, 'Delta H (last 2 iters)', DeltaH)
 
-    # Calculating a dataframe showing the evolution of temperature and H2O vs. number of iterations
     Iter=It_sample
-    for i in range(0, len(Dupliqs)):
-        if i==0:
-            T_evol=pd.DataFrame(data={'Iteration': Iter, 'Sample_0_T_calc': T_sample[0, :]})
-            T_evol.insert(i+1, 'Sample_0_H_calc', H2O_sample[0, :])
-        else:
-            new_col_name_T=('Sample_'+str(i)+ "_T_calc")
-            new_col_name_H=('Sample_'+str(i)+ "_H_calc")
-            T_evol.insert(2*i, new_col_name_T, T_sample[i, :])
-            T_evol.insert(2*i+1, new_col_name_H, H2O_sample[i, :])
+
+    df_t=pd.DataFrame(T_sample.T)
+    df_Temp=df_t.add_prefix('Sample_')
+    df_Temp2=df_Temp.add_suffix('_T_calc')
+    df_Temp2
+
+
+
+    df_h=pd.DataFrame(H2O_sample.T)
+    df_H2O=df_h.add_prefix('Sample_')
+    df_H2O2=df_H2O.add_suffix('_T_calc')
+    df_H2O2
+
+    T_evol=pd.concat([df_Temp2, df_H2O2], axis=1)
+    T_evol['Iteration']=Iter
+    # Calculating a dataframe showing the evolution of temperature and H2O vs. number of iterations
+    # Iter=It_sample
+    # for i in range(0, len(Dupliqs)):
+    #     if i==0:
+    #         T_evol=pd.DataFrame(data={'Iteration': Iter, 'Sample_0_T_calc': T_sample[0, :]})
+    #         T_evol.insert(i+1, 'Sample_0_H_calc', H2O_sample[0, :])
+    #     else:
+    #         new_col_name_T=('Sample_'+str(i)+ "_T_calc")
+    #         new_col_name_H=('Sample_'+str(i)+ "_H_calc")
+    #         T_evol.insert(2*i, new_col_name_T, T_sample[i, :])
+    #         T_evol.insert(2*i+1, new_col_name_H, H2O_sample[i, :])
+    #
+    #
+    #
+    #
+
+    if Ab_An_P2008 is True:
+        print('Applying filter to only average those that pass the An-Ab eq test of Putirka, 2008')
+        Combo_fspar_liqs2=Combined_output.loc[Combined_output['Pass An-Ab Eq Test Put2008?'].str.contains('Yes')].reset_index(drop=True)
+
+    if Ab_An_P2008 is False:
+        print('We are returning all pairs, if you want to use the Ab-An equilibrium test of Putirka (2008), enter Ab_An_P2008=True to filter prior to averaging')
+        Combo_fspar_liqs2=Combined_output
+
+
+
+
+    FsparNumbers = Combo_fspar_liqs2['ID_Fspar'].unique()
+
+    Combo_fspar_liqs3=Combo_fspar_liqs2.drop(['Pass An-Ab Eq Test Put2008?', 'T_K_calc'], axis=1)
+    Combo_fspar_liqs3['T_K_calc']=Combo_fspar_liqs2['T_K_calc'].astype(float)
+    if len(FsparNumbers) > 0:
+        if plag_comps is not None:
+            df1_Mean_nopref=Combo_fspar_liqs3.groupby(['ID_Fspar', 'Sample_ID_Plag'], as_index=False).mean()
+            df1_Std_nopref=Combo_fspar_liqs3.groupby(['ID_Fspar', 'Sample_ID_Plag'], as_index=False).std()
+            count=Combo_fspar_liqs2.groupby('ID_Fspar',as_index=False).count().iloc[:, 1]
+            df1_Mean_nopref['# of Liqs Averaged']=count
+
+            Sample_ID_Fspar_Mean=df1_Mean_nopref['Sample_ID_Plag']
+            Sample_ID_Fspar_Std=df1_Std_nopref['Sample_ID_Plag']
+            df1_Mean=df1_Mean_nopref.add_prefix('Mean_')
+            df1_Std=df1_Std_nopref.add_prefix('Std_')
+
+            df1_Mean.rename(columns={"Mean_ID_Fspar": "ID_Fspar"}, inplace=True)
+            df1_Mean.rename(columns={"Mean_# of Liqs Averaged": "# of Liqs Averaged"}, inplace=True)
+            df1_Std.rename(columns={"Std_ID_Fspar": "ID_Fspar"}, inplace=True)
+
+
+
+            df1_M=pd.merge(df1_Mean, df1_Std, on=['ID_Fspar'])
+            df1_M['Sample_ID_Plag']=Sample_ID_Fspar_Mean
+
+    else:
+        raise Exception(
+            'No Matches - to set less strict filters, change our Kd filter')
+
+
+    print('Done!!! I found a total of N='+str(len(Combo_fspar_liqs3)) + ' Fspar-Liq matches using the specified filter. N=' + str(len(df1_M)) + ' Fspar out of the N='+str(LenFspar)+' Fspar that you input matched to 1 or more liquids')
+
+    return {'Av_HTs': df1_M, 'All_HTs': Combo_fspar_liqs2, 'T_H_Evolution':  T_evol}
+    # Now we do the averaging step for each feldspar crystal
 
 
 
 
-    return {'T_H_calc': Combined_output, 'T_H_Evolution':  T_evol}
 
 
 
 
 def calculate_fspar_liq_temp_matching(*, liq_comps, plag_comps=None,
 kspar_comps=None, H2O_Liq=None, equationT=None,
-P=None, eq_tests=False):
+P=None, Ab_An_P2008=False):
     '''
     Evaluates all possible Plag-liq or kspar-liq pairs,
     returns T (K) and equilibrium test values. Users must investigate correct values for eq tests.
@@ -1289,11 +1496,15 @@ P=None, eq_tests=False):
     P: float, int, pandas.Series
         Pressure in kbar to perform calculations at.
 
+
     Returns
     -------
-    pandas.DataFrame
-        T in K for all posible plag-liq matches, along with equilibrium
-        tests, components and input mineral compositions
+    dict
+        'Av_PTs': df of averaged T for each Plag, and all the liquids it matches (+eq tests etc)
+        'All_PTs: df of all T for all possible Plag-Liq combinations (+eq tests etc)
+
+
+
     '''
 
 
@@ -1345,7 +1556,7 @@ P=None, eq_tests=False):
 
 
     # Adding an ID label to help with melt-liq rematching later
-    cat_fspar['ID_Plag'] = cat_fspar.index
+    cat_fspar['ID_Fspar'] = cat_fspar.index
     cat_liqs['ID_liq'] = cat_liqs.index.astype('float64')
     if "Sample_ID_Plag" not in cat_fspar:
         cat_fspar['Sample_ID_Plag'] = cat_fspar.index.astype('float64')
@@ -1369,8 +1580,12 @@ P=None, eq_tests=False):
     # Combo_plags_liqs_1['K_Barth'] = Combo_plags_liqs_1['Ab_liq'] / \
     #     Combo_plags_liqs_1['Ab_Plag']
     LenCombo = str(np.shape(Combo_fspar_liqs)[0])
-    print("Considering " + LenCombo +
-          " liq-spar pairs, be patient if this is >>1 million!")
+
+    LenFspar=len(cat_fspar)
+    LenLiqs=len(liq_comps)
+    print("Considering N=" + str(LenFspar) + " Fspar & N=" + str(LenLiqs) +" Liqs, which is a total of N="+ str(LenCombo) +
+          " Liq-Fspar pairs, be patient if this is >>1 million!")
+
 
     if plag_comps is not None:
         T_K_calc=calculate_fspar_liq_temp(meltmatch_plag=Combo_fspar_liqs_1,
@@ -1382,21 +1597,104 @@ P=None, eq_tests=False):
         equationT=equationT, P=P)
         Combo_fspar_liqs_1.insert(0, "T_K_calc", T_K_calc)
 
-    if eq_tests is False:
-        return Combo_fspar_liqs_1
-    if eq_tests is True:
-        if kspar_comps is not None:
-            print('Sorry, no equilibrium tests implemented for Kspar-Liquid. Weve just returned the major elements')
-            Combo_fspar_liqs.insert(0, "T_K_calc", T_K_calc)
-            return Combo_fspar_liqs
 
+    if kspar_comps is not None:
+        print('Sorry, no equilibrium tests implemented for Kspar-Liquid. Weve returned all possible pairs, you will have to filter them yourselves')
+        Combo_fspar_liqs.insert(0, "T_K_calc", T_K_calc)
+        Combo_fspar_liqs2=Combo_fspar_liqs
+
+
+
+    if plag_comps is not None:
+        eq_tests=calculate_plag_liq_eq_tests(meltmatch=Combo_fspar_liqs_1,
+        P=P, T=T_K_calc)
+
+
+        cols_to_move = ['T_K_calc']
+        eq_testsN = eq_tests[cols_to_move + [
+            col for col in eq_tests.columns if col not in cols_to_move]]
+        if Ab_An_P2008 is True:
+            Combo_fspar_liqs2=eq_testsN.loc[eq_testsN['Pass An-Ab Eq Test Put2008?'].str.contains('Yes')].reset_index(drop=True)
+            print('Applying filter to only average those that pass the An-Ab eq test of Putirka, 2008')
+
+        if Ab_An_P2008 is False:
+            print('We are returning all pairs, if you want to use the Ab-An equilibrium test of Putirka (2008), enter Ab_An_P2008=True')
+            Combo_fspar_liqs2=eq_testsN
+
+        Combo_fspar_liqs3=Combo_fspar_liqs2.drop(['Pass An-Ab Eq Test Put2008?', 'T_K_calc'], axis=1)
+
+    else:
+        Combo_fspar_liqs3=Combo_fspar_liqs_1
+
+
+
+
+    FsparNumbers = Combo_fspar_liqs2['ID_Fspar'].unique()
+
+
+    Combo_fspar_liqs3['T_K_calc']=Combo_fspar_liqs2['T_K_calc'].astype(float)
+    if len(FsparNumbers) > 0:
         if plag_comps is not None:
-            eq_tests=calculate_plag_liq_eq_tests(meltmatch=Combo_fspar_liqs_1,
-            P=P, T=T_K_calc)
+            df1_Mean_nopref=Combo_fspar_liqs3.groupby(['ID_Fspar', 'Sample_ID_Plag'], as_index=False).mean()
+            df1_Std_nopref=Combo_fspar_liqs3.groupby(['ID_Fspar', 'Sample_ID_Plag'], as_index=False).std()
+            count=Combo_fspar_liqs2.groupby('ID_Fspar',as_index=False).count().iloc[:, 1]
+            df1_Mean_nopref['# of Liqs Averaged']=count
+
+            Sample_ID_Fspar_Mean=df1_Mean_nopref['Sample_ID_Plag']
+            Sample_ID_Fspar_Std=df1_Std_nopref['Sample_ID_Plag']
+            df1_Mean=df1_Mean_nopref.add_prefix('Mean_')
+            df1_Std=df1_Std_nopref.add_prefix('Std_')
+
+            df1_Mean.rename(columns={"Mean_ID_Fspar": "ID_Fspar"}, inplace=True)
+            df1_Mean.rename(columns={"Mean_# of Liqs Averaged": "# of Liqs Averaged"}, inplace=True)
+            df1_Std.rename(columns={"Std_ID_Fspar": "ID_Fspar"}, inplace=True)
+
+
+
+            df1_M=pd.merge(df1_Mean, df1_Std, on=['ID_Fspar'])
+            df1_M['Sample_ID_Plag']=Sample_ID_Fspar_Mean
+
+
+
+            # cols_to_move = ['Sample_ID_Plag',
+            #                 'Mean_T_K_calc', 'Std_T_K_calc']
+            #
+            # df1_M = df1_M[cols_to_move +
+            #             [col for col in df1_M.columns if col not in cols_to_move]]
+
+
+
+        if kspar_comps is not None:
+            df1_Mean_nopref=Combo_fspar_liqs3.groupby(['ID_Fspar', 'Sample_ID_Kspar'], as_index=False).mean()
+            df1_Std_nopref=Combo_fspar_liqs3.groupby(['ID_Fspar', 'Sample_ID_Kspar'], as_index=False).std()
+            count=Combo_fspar_liqs2.groupby('ID_Fspar',as_index=False).count().iloc[:, 1]
+            df1_Mean_nopref['# of Liqs Averaged']=count
+
+            Sample_ID_Fspar_Mean=df1_Mean_nopref['Sample_ID_Kspar']
+            Sample_ID_Fspar_Std=df1_Std_nopref['Sample_ID_Kspar']
+            df1_Mean=df1_Mean_nopref.add_prefix('Mean_')
+            df1_Std=df1_Std_nopref.add_prefix('Std_')
+
+            df1_Mean.rename(columns={"Mean_ID_Fspar": "ID_Fspar"}, inplace=True)
+            df1_Mean.rename(columns={"Mean_# of Liqs Averaged": "# of Liqs Averaged"}, inplace=True)
+            df1_Std.rename(columns={"Std_ID_Fspar": "ID_Fspar"}, inplace=True)
+
+
+
+            df1_M=pd.merge(df1_Mean, df1_Std, on=['ID_Fspar'])
+            df1_M['Sample_ID_Kspar']=Sample_ID_Fspar_Mean
+
+
+    else:
+        raise Exception(
+            'No Matches - to set less strict filters, change our Kd filter')
+
+
+    print('Done!!! I found a total of N='+str(len(Combo_fspar_liqs3)) + ' Fspar-Liq matches using the specified filter. N=' + str(len(df1_M)) + ' Fspar out of the N='+str(LenFspar)+' Fspar that you input matched to 1 or more liquids')
+
+    return {'Av_PTs': df1_M, 'All_PTs': Combo_fspar_liqs2}
+    # Now we do the averaging step for each feldspar crystal
+
 
-            cols_to_move = ['T_K_calc']
-            eq_tests = eq_tests[cols_to_move + [
-                col for col in eq_tests.columns if col not in cols_to_move]]
-        return eq_tests
 
 
