@@ -19,6 +19,21 @@ def garnet_CARP_class_Griffin2002(gt_comps):
     gt_comps: garnet composition dataframe imported from core function
     '''
 
+    idx_critical_missing = []
+    for i in range(0,len(gt_comps)):
+
+        if (pd.isna(gt_comps['CaO_Gt'][i]) == True) or (pd.isna(gt_comps['Cr2O3_Gt'][i]) == True) or\
+         (pd.isna(gt_comps['MgO_Gt'][i]) == True) or (pd.isna(gt_comps['FeOt_Gt'][i]) == True) or\
+         (pd.isna(gt_comps['TiO2_Gt'][i]) == True) or  (pd.isna(gt_comps['Zr_Gt'][i]) == True) or\
+         (pd.isna(gt_comps['Y_Gt'][i]) == True):
+
+            idx_critical_missing.append(i)
+
+    if len(idx_critical_missing) > 0:
+        print('WARNING')
+        print('There are critical components missing in some of the samples for CARP analysis and might lead to wrong analysis. idx list: ')
+        print(idx_critical_missing)
+
     h1 = []
     h2 = []
     h3 = []
@@ -207,17 +222,40 @@ def garnet_CARP_class_Griffin2002(gt_comps):
 
     return carp_depleted_harz, carp_depleted_lherz, carp_depleted_metasomatised, carp_fertile_lherz, carp_melt_metas, carp_unclass, len_tot
 
-def garnet_class_Grutter2003(gt_comps):
+def garnet_class_Grutter2004(gt_comps):
 
     '''
     A function to classify Cr-pyrope garnet xenocrysts according to
-    the study of Grutter et al. (2003)
-    :cite: `grutter2003`
+    the study of Grutter et al. (2004)
+    :cite: `grutter2004`
 
     ###Parameters###
     gt_comps: garnet composition dataframe imported from core function
     '''
 
+    idx_critical_missing = []
+    idx_na_missing = []
+    for i in range(0,len(gt_comps)):
+        if (pd.isna(gt_comps['CaO_Gt'][i]) == True) or (pd.isna(gt_comps['Cr2O3_Gt'][i]) == True) or\
+         (pd.isna(gt_comps['MgO_Gt'][i]) == True) or (pd.isna(gt_comps['FeOt_Gt'][i]) == True) or\
+         (pd.isna(gt_comps['MnO_Gt'][i]) == True) or  (pd.isna(gt_comps['TiO2_Gt'][i]) == True):
+
+            idx_critical_missing.append(i)
+
+        if (pd.isna(gt_comps['Na2O_Gt'][i]) == True):
+
+            idx_na_missing.append(i)
+
+    if len(idx_critical_missing) > 0:
+        print('WARNING')
+        print('There are critical components missing in some of the samples for Grutter2004 analysis. For these samples analyses might be wrong. idx list: ')
+        print(idx_critical_missing)
+
+    if len(idx_na_missing) > 0:
+        print('WARNING')
+        print('Na2O component is missing in some of the samples, diamondiferious classificatons might be wrong, defaulting to non-indicator classification. idx list: ')
+        print(idx_na_missing)
+
     ca_int = []
     mg_num_g = []
     g10 = []
@@ -375,6 +413,15 @@ def garnet_ca_cr_class_Griffin2002(gt_comps):
     gt_comps: garnet composition dataframe imported from core function
     '''
 
+    #Checking if any CaO or Cr2O3 is missing
+
+    for i in range(0,len(gt_comps)):
+
+        if (pd.isna(gt_comps['CaO_Gt'][i]) == True) or (pd.isna(gt_comps['Cr2O3_Gt'][i]) == True):
+
+            print('WARNING')
+            print('There is CaO or Cr2O3 missing in one of the samples. idx = ' + str(i))
+
     low_cr_ca = [0,9,9,0]
     low_cr_cr = [0,0,1.52,1.52]
 
@@ -534,7 +581,7 @@ def y_zr_classification_Griffin2002(gt_comps):
 
     return yzr_class
 
-def calculate_ol_mg(gt_comps, T_Ni, xMg, xCa, xFe):
+def calculate_ol_mg(gt_comps, T_Ni):
 
     '''
     A function to derive olivine Mg# from coxisting Cr-pyrope garnet xenocrysts.
@@ -544,10 +591,21 @@ def calculate_ol_mg(gt_comps, T_Ni, xMg, xCa, xFe):
     gt_comps: garnet composition dataframe imported from core function
     T_Ni: Temperature output taken from one of the garnet thermometry methods in
     Kelvin.
+
+    Returns
+    --------------
+    Numpy array
+        Ol Mg#
     '''
 
     mg_ol = np.zeros(len(gt_comps['MgO_Gt']))
 
+    gt_calc = calculate_garnet_components(gt_comps = gt_comps)
+
+    xMg = np.array(gt_calc['Mg_MgFeCa_Gt'])
+    xCa = np.array(gt_calc['Ca_MgFeCa_Gt'])
+    xFe = np.array(gt_calc['Fe_MgFeCa_Gt'])
+
     T_Ni = np.array(T_Ni) - 273.0 #converting to celsius for calculations
 
     for i in range(0,len(np.array(gt_comps['MgO_Gt']))):
@@ -579,6 +637,11 @@ def calculate_al2o3_whole_rock(gt_comps):
     ###Parameters###
     gt_comps: garnet composition dataframe imported from core function
 
+    Returns
+    --------------
+    Numpy array
+        Whole rock Al2O3 values.
+
     '''
 
     wr_al_y = np.zeros(len(gt_comps['Y_Gt']))
