@@ -116,7 +116,7 @@ def import_lepr_file(filename):
     pandas DataFrames stored in a dictionary. E.g., Access Cpxs using output.Cpxs
         my_input = pandas dataframe of the entire spreadsheet
         mylabels = sample labels
-        Experimental_press_temp = User-entered PT
+        Experimental_PT = User-entered PT
         Fluid=pandas dataframe of fluid compositions
         Liqs=pandas dataframe of liquid oxides
         Ols=pandas dataframe of olivine oxides
@@ -136,7 +136,10 @@ def import_lepr_file(filename):
     if "Experiment" in sheet_names:
         my_input_Exp = pd.read_excel(filename, sheet_name="Experiment")
         myExp = pd.DataFrame(data={'Citation': my_input_Exp['Citation'], 'Experiment': my_input_Exp['Experiment'],
-                                   'T_K': my_input_Exp['T (C)'] + 273.15, 'P_kbar': 10 * my_input_Exp['P (GPa)']}
+                                   'T_K': my_input_Exp['T (C)']+273.15, 'P_kbar': 10*my_input_Exp['P (GPa)'],
+                                   'Duration': my_input_Exp['Duration (hours)'],
+                                   'Laboratory': my_input_Exp['Laboratory']
+                                   }
                              )
     if "Experiment" not in sheet_names:
         myExp = pd.DataFrame()
@@ -156,7 +159,7 @@ def import_lepr_file(filename):
         my_input_Plag = pd.read_excel(
             filename, sheet_name="Plagioclase").fillna(0)
         my_input_Plag['FeOT value'] = my_input_Plag['FeO value'] + \
-            0.89998 * my_input_Plag['Fe2O3 value']
+            0.89998*my_input_Plag['Fe2O3 value']
         myPlag = pd.DataFrame(data={'Experiment': my_input_Plag['Experiment'],
                                     'SiO2_Plag': my_input_Plag['SiO2 value'],
                                     'TiO2_Plag': my_input_Plag['TiO2 value'],
@@ -178,21 +181,29 @@ def import_lepr_file(filename):
         my_input_Cpx = pd.read_excel(
             filename, sheet_name="Clinopyroxene").fillna(0)
         my_input_Cpx['FeOT value'] = my_input_Cpx['FeO value'] + \
-            0.89998 * my_input_Cpx['Fe2O3 value']
+            0.89998*my_input_Cpx['Fe2O3 value']
         myCpx = pd.DataFrame(data={'Experiment': my_input_Cpx['Experiment'],
                                    'SiO2_Cpx': my_input_Cpx['SiO2 value'],
                                    'TiO2_Cpx': my_input_Cpx['TiO2 value'],
+                                   'TiO2_Cpx_Err': my_input_Cpx['TiO2 error'],
                                    'Al2O3_Cpx': my_input_Cpx['Al2O3 value'],
+                                   'Al2O3_Cpx_Err': my_input_Cpx['Al2O3 error'],
                                    'FeOt_Cpx': my_input_Cpx['FeOT value'],
                                    'MnO_Cpx': my_input_Cpx['MnO value'],
                                    'MgO_Cpx': my_input_Cpx['MgO value'],
                                    'CaO_Cpx': my_input_Cpx['CaO value'],
                                    'Na2O_Cpx': my_input_Cpx['Na2O value'],
+                                    'Na2O_Cpx_Err': my_input_Cpx['Na2O error'],
                                    'K2O_Cpx': my_input_Cpx['K2O value'],
                                    'Cr2O3_Cpx': my_input_Cpx['Cr2O3 value'],
                                    'P2O5_Cpx': my_input_Cpx['P2O5 value']})
+        if 'n' in my_input_Cpx.columns:
+            myCpx['N_meas_Cpx']=my_input_Cpx['n']
+        if 'Number of analyses' in my_input_Cpx.columns:
+            myCpx['N_meas_Cpx']=my_input_Cpx['Number of analyses']
         myCpx_Exp = pd.merge(myCpx, myExp, on="Experiment")
 
+
     if "Clinopyroxene" not in sheet_names:
         myCpx = pd.DataFrame()
         myCpx_Exp = pd.DataFrame()
@@ -201,7 +212,7 @@ def import_lepr_file(filename):
         my_input_Opx = pd.read_excel(
             filename, sheet_name="Orthopyroxene").fillna(0)
         my_input_Opx['FeOT value'] = my_input_Opx['FeO value'] + \
-            0.89998 * my_input_Opx['Fe2O3 value']
+            0.89998*my_input_Opx['Fe2O3 value']
         myOpx = pd.DataFrame(data={'Experiment': my_input_Opx['Experiment'],
                                    'SiO2_Opx': my_input_Opx['SiO2 value'],
                                    'TiO2_Opx': my_input_Opx['TiO2 value'],
@@ -219,10 +230,11 @@ def import_lepr_file(filename):
     if "Orthopyroxene" not in sheet_names:
         myOpx = pd.DataFrame()
         myOpx_Exp = pd.DataFrame()
+
     if "Liquid" in sheet_names:
         my_input_Liq = pd.read_excel(filename, sheet_name="Liquid").fillna(0)
         my_input_Liq['FeOT value'] = my_input_Liq['FeO value'] + \
-            0.89998 * my_input_Liq['Fe2O3 value']
+            0.89998*my_input_Liq['Fe2O3 value']
         myLiq = pd.DataFrame(data={'Experiment': my_input_Liq['Experiment'],
                                    'SiO2_Liq': my_input_Liq['SiO2 value'],
                                    'TiO2_Liq': my_input_Liq['TiO2 value'],
@@ -235,7 +247,11 @@ def import_lepr_file(filename):
                                    'K2O_Liq': my_input_Liq['K2O value'],
                                    'Cr2O3_Liq': my_input_Liq['Cr2O3 value'],
                                    'P2O5_Liq': my_input_Liq['P2O5 value'],
-                                   'H2O_Liq': my_input_Liq['H2O value']})
+                                   'H2O_Liq': my_input_Liq['H2O value'],
+                                   'Total': my_input_Liq['total value']})
+        if 'n' in my_input_Liq.columns:
+            myLiq['N_meas_Liq']=my_input_Liq['n']
+
         myLiq_Exp = pd.merge(myLiq, myExp, on="Experiment")
     if "Liquid" not in sheet_names:
         myLiq = pd.DataFrame()
@@ -244,7 +260,7 @@ def import_lepr_file(filename):
         my_input_Amp = pd.read_excel(
             filename, sheet_name="Amphibole").fillna(0)
         my_input_Amp['FeOT value'] = my_input_Amp['FeO value'] + \
-            0.89998 * my_input_Amp['Fe2O3 value']
+            0.89998*my_input_Amp['Fe2O3 value']
         myAmp = pd.DataFrame(data={'Experiment': my_input_Amp['Experiment'],
                                    'SiO2_Amp': my_input_Amp['SiO2 value'],
                                    'TiO2_Amp': my_input_Amp['TiO2 value'],
@@ -266,7 +282,7 @@ def import_lepr_file(filename):
     if "Olivine" in sheet_names:
         my_input_Ol = pd.read_excel(filename, sheet_name="Olivine").fillna(0)
         my_input_Ol['FeOT value'] = my_input_Ol['FeO value'] + \
-            0.89998 * my_input_Ol['Fe2O3 value']
+            0.89998*my_input_Ol['Fe2O3 value']
         myOl = pd.DataFrame(data={'Experiment': my_input_Ol['Experiment'],
                             'SiO2_Ol': my_input_Ol['SiO2 value'],
                                   'TiO2_Ol': my_input_Ol['TiO2 value'],
@@ -285,9 +301,8 @@ def import_lepr_file(filename):
         myOl = pd.DataFrame()
         myOl_Exp = pd.DataFrame()
 
-    return {'Experimental_press_temp': myExp, 'Liquids': myLiq, 'Liqs_Exp': myLiq_Exp, 'Fluids': myFluid, 'Fluids_Exp': myFluid_Exp, 'Plags': myPlag, 'Plags_Exp': myPlag_Exp,
-            'Cpxs': myCpx, 'Cpxs_Exp': myCpx_Exp, 'Opxs': myOpx, 'Opxs_Exp': myOpx_Exp, 'Amps': myAmp, 'Amps_Exp': myAmp_Exp, 'Ols_Exp': myOl_Exp}
-    # 'Experimental_press_temp':Experimental_press_temp1, 'Cpxs': myCPXs1, 'Opxs': myOPXs1, 'Liqs':myLiquids1, 'Plags': myPlags1, 'Kspars': myKspars1,'Amps': myAmphs1, 'Ols': myOls1, 'Sps': mySps1}#,
+    return {'Experimental_PT': myExp, 'Liquids': myLiq, 'Liqs_Exp': myLiq_Exp, 'Fluids': myFluid, 'Fluids_Exp': myFluid_Exp, 'Plags': myPlag, 'Plags_Exp': myPlag_Exp,
+            'Cpxs': myCpx,  'Cpxs_Exp': myCpx_Exp, 'Opxs': myOpx, 'Opxs_Exp': myOpx_Exp, 'Amps': myAmp, 'Amps_Exp': myAmp_Exp, 'Ols_Exp': myOl_Exp}
 
 
 # Loading Excel, returns a disctionry
@@ -331,6 +346,8 @@ def import_excel(filename, sheet_name, sample_label=None, GEOROC=False, suffix=N
             my_input = pd.read_excel(filename)
             #my_input[my_input < 0] = 0
 
+    if any(my_input.columns.str.startswith(' ')):
+        w.warn('your input file has some columns that start with spaces. This could cause you big problems if they are at the start of oxide names. Please ammend your file and reload.')
     if suffix is not None:
         if any(my_input.columns.str.contains(suffix)):
             w.warn('We notice you have specified a suffix, but some of your columns already have this suffix. '
@@ -380,14 +397,20 @@ def import_excel(filename, sheet_name, sample_label=None, GEOROC=False, suffix=N
             " where you can also enter a Fe3Fet_Liq heading used for equilibrium tests")
 
     if any(my_input.columns.str.contains("FeO_")) and (all(my_input.columns.str.contains("FeOt_")==False)):
-        raise ValueError("No FeOt found. You've got a column heading with FeO. To avoid errors based on common EPMA outputs"
+
+        if any(my_input.columns.str.contains("FeO_Liq")) and any(my_input.columns.str.contains("Fe2O3_Liq")):
+            my_input_c['FeOt_Liq']=my_input_c['FeO_Liq']+my_input_c['Fe2O3_Liq']*0.89998
+
+
+        else:
+            raise ValueError("No FeOt found. You've got a column heading with FeO. To avoid errors based on common EPMA outputs"
         " thermobar only recognises columns with FeOt for all phases except liquid"
         " where you can also enter a Fe3Fet_Liq heading used for equilibrium tests")
 
-    if any(my_input.columns.str.contains("Fe2O3_")) and (all(my_input.columns.str.contains("FeOt_")==False)):
-        raise ValueError("No FeOt column found. You've got a column heading with Fe2O3. To avoid errors based on common EPMA outputs"
-        " thermobar only recognises columns with FeOt for all phases except liquid"
-        " where you can also enter a Fe3Fet_Liq heading used for equilibrium tests")
+    # if any(my_input.columns.str.contains("Fe2O3_")) and (all(my_input.columns.str.contains("FeOt_")==False)):
+    #     raise ValueError("No FeOt column found. You've got a column heading with Fe2O3. To avoid errors based on common EPMA outputs"
+    #     " thermobar only recognises columns with FeOt for all phases except liquid"
+    #     " where you can also enter a Fe3Fet_Liq heading used for equilibrium tests")
 
     if any(my_input.columns.str.contains("FeOT_")) and (all(my_input.columns.str.contains("FeOt_")==False)):
         raise ValueError("No FeOt column found. You've got a column heading with FeOT. Change to a lower case t")
@@ -441,6 +464,11 @@ def import_excel(filename, sheet_name, sample_label=None, GEOROC=False, suffix=N
     myAmphs1 = my_input_c.reindex(df_ideal_amp.columns, axis=1).fillna(0)
     myAmphs1 = myAmphs1.apply(pd.to_numeric, errors='coerce').fillna(0)
     myAmphs1[myAmphs1 < 0] = 0
+
+    myGts1 = my_input_c.reindex(df_ideal_gt.columns, axis=1).fillna(0)
+    myGts1 = myGts1.apply(pd.to_numeric, errors='coerce').fillna(0)
+    myGts1[myGts1 < 0] = 0
+
     # Adding sample Names
     if "Sample_ID_Cpx" in my_input_c:
         myCPXs1['Sample_ID_Cpx'] = my_input_c['Sample_ID_Cpx']
@@ -467,6 +495,11 @@ def import_excel(filename, sheet_name, sample_label=None, GEOROC=False, suffix=N
     else:
         myAmphs1['Sample_ID_Amp'] = my_input.index
 
+    if "Sample_ID_Gt" in my_input_c:
+        myGts1['Sample_ID_Gt'] = my_input_c['Sample_ID_Gt']
+    else:
+        myGts1['Sample_ID_Gt'] = my_input.index
+
     if "Sample_ID_Ol" in my_input_c:
         myOls1['Sample_ID_Ol'] = my_input_c['Sample_ID_Ol']
     else:
@@ -502,7 +535,7 @@ def import_excel(filename, sheet_name, sample_label=None, GEOROC=False, suffix=N
     #     mySps1['T_K'] = my_input['T_K']
     #     myLiquids1['T_K'] = my_input['T_K']
 
-    return {'my_input': my_input, 'my_oxides': myOxides1, 'Experimental_press_temp': Experimental_press_temp1, 'Cpxs': myCPXs1, 'Opxs': myOPXs1, 'Liqs': myLiquids1,
+    return {'my_input': my_input, 'my_oxides': myOxides1, 'Experimental_press_temp': Experimental_press_temp1, 'Cpxs': myCPXs1, 'Opxs': myOPXs1, 'Liqs': myLiquids1, 'Gts': myGts1,
             'Plags': myPlags1, 'Kspars': myKspars1, 'Amps': myAmphs1, 'Ols': myOls1, 'Sps': mySps1}  # , 'y1': y1 ,'y2': y2}
 
 
@@ -554,6 +587,32 @@ def import_excel_errors(filename, sheet_name, GEOROC=False):
 # E.g., if users only enter FeO (not FeOt and Fe2O3), allocates a FeOt
 # column. If enter FeO and Fe2O3, put a FeOt column
 
+# Give warnings if no FeOt
+    cols=my_input.columns
+    if any(cols.str.startswith(" ")):
+        w.warn('We have found some spaces  at the start of your column headings. Check these arent oxide headings, it wont read them')
+    if ("FeOt_Cpx" not in my_input) and (any(cols.str.contains('_Cpx'))):
+        w.warn('No FeOt_Cpx column, please check of you wanted any Fe in calcs')
+
+    if ("FeOt_Opx" not in my_input) and (any(cols.str.contains('_Opx'))):
+        w.warn('No FeOt_Opx column, please check of you wanted any Fe in calcs')
+
+    if ("FeOt_Amp" not in my_input) and (any(cols.str.contains('_Amp'))):
+        w.warn('No FeOt_Amp column, please check of you wanted any Fe in calcs')
+
+    if ("FeOt_Sp" not in my_input) and (any(cols.str.contains('_Sp'))):
+        w.warn('No FeOt_Sp column, please check of you wanted any Fe in calcs')
+
+
+    if ("FeOt_Ol" not in my_input) and (any(cols.str.contains('_Ol'))):
+        w.warn('No FeOt_Ol column, please check of you wanted any Fe in calcs')
+
+    if ("FeOt_Plag" not in my_input) and (any(cols.str.contains('_Plag'))):
+        w.warn('No FeOt_Plag column, please check of you wanted any Fe in calcs')
+
+    if ("FeOt_Kspar" not in my_input) and (any(cols.str.contains('_Kspar'))):
+        w.warn('No FeOt_Kspar column, please check of you wanted any Fe in calcs')
+
     if "FeO_Cpx_Err" in my_input and "FeOt_Cpx" not in my_input and "Fe2O3_Cpx" not in my_input:
         my_input_c['FeOt_Cpx_Err'] = my_input_c['FeO_Cpx_Err']
         print('Only FeO_Cpx found in input, the code has allocated this to FeOt_Cpx')
@@ -702,14 +761,22 @@ def import_excel_errors(filename, sheet_name, GEOROC=False):
         mySps1['T_K_Err'] = my_input['T_K_Err']
         myLiquids1['T_K_Err'] = my_input['T_K_Err']
 
-    return {'my_input_Err': my_input, 'Experimental_press_temp_Err': Experimental_press_temp1, 'Cpxs_Err': myCPXs1,
-            'Opxs_Err': myOPXs1, 'Liqs_Err': myLiquids1, 'Plags_Err': myPlags1, 'Kspars_Err': myKspars1, 'Amps_Err': myAmphs1, 'Ols_Err': myOls1, 'Sps_Err': mySps1}
+    return {'my_input_Err': my_input,
+    'Experimental_press_temp_Err': Experimental_press_temp1,
+    'Cpxs_Err': myCPXs1,
+    'Opxs_Err': myOPXs1,
+    'Liqs_Err': myLiquids1,
+    'Plags_Err': myPlags1,
+    'Kspars_Err': myKspars1,
+    'Amps_Err': myAmphs1,
+    'Ols_Err': myOls1,
+    'Sps_Err': mySps1}
 
 
 # Gets liquid dataframe into a format that can be used in VESical. Have to
 # have VESIcal installed for the final step, which we do in the script for
 # simplicity
-def convert_to_vesical(liq_comps, T1, Fe3Fet_Liq=None):
+def convert_to_vesical(liq_comps, T1, unit='Kelvin', Fe3Fet_Liq=None):
     ''' Takes liquid dataframe in the format used for PyMME, and strips the _Liq string so that it can be input into VESical. Also removes the Fe3FeTcolumn, and appends temperature (converted from Kevlin to celcius)
 
 
@@ -719,7 +786,10 @@ def convert_to_vesical(liq_comps, T1, Fe3Fet_Liq=None):
         DataFrame of liquid compositions.
 
     T1: Panda series, int, float
-        Temperature in Kelvin (e.g., from a thermometer of choice)
+        Temperature in Kelvin by default (e.g., from a thermometer of choice)
+
+    unit: Kelvin or Celcius
+        What unit supplied temperature is in
 
    Returns
     -------
@@ -727,7 +797,10 @@ def convert_to_vesical(liq_comps, T1, Fe3Fet_Liq=None):
 
     '''
     df = liq_comps.copy()
-    df['Temp'] = T1 - 273.15
+    if unit=='Kelvin':
+        df['Temp'] = T1 - 273.15
+    else:
+        df['Temp'] = T1
     if Fe3Fet_Liq is None:
         Fe3Fet_Liq=df['Fe3Fet_Liq']
     FeOt=df['FeOt_Liq']
